@@ -4,7 +4,7 @@ import sys
 import commands
 import gettext
 
-gettext.install("live-installer", "/usr/share/linuxmint/locale")
+gettext.install("live-installer", "/usr/share/locale")
 
 sys.path.insert(1, '/usr/lib/live-installer')
 from frontend.gtk_interface import InstallerWindow
@@ -15,8 +15,9 @@ from gi.repository import Gtk
 
 # main entry
 if __name__ == "__main__":
-	if("install" in commands.getoutput("cat /proc/cmdline")):
-		win = InstallerWindow(fullscreen=True)
+
+	if ("--expert-mode" in sys.argv):
+		win = InstallerWindow(expert_mode=True)
 	else:
-		win = InstallerWindow(fullscreen=False)
+		win = InstallerWindow()
 	Gtk.main()
