@@ -8,13 +8,21 @@ import re
 import sys
 import subprocess
 from collections import defaultdict
-
-from gi.repository import Gtk, Gdk
+import gi
+gi.require_version('Gtk', '3.0')
+from gi.repository import Gtk, Gdk, GObject
 import parted
 import commands
 import gettext
+import time
 
-gettext.install("live-installer", "/usr/share/linuxmint/locale")
+gettext.install("live-installer", "/usr/share/locale")
+
+# Used as a decorator to run things in the main loop, from another thread
+def idle(func):
+    def wrapper(*args, **kwargs):
+        GObject.idle_add(func, *args, **kwargs)
+    return wrapper
 
 def shell_exec(command):
     return subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
@@ -57,6 +65,41 @@ with open(RESOURCE_DIR + 'disk-partitions.html') as f:
     DISK_TEMPLATE = re.sub('<style>[\s\S]+?</style>', lambda match: match.group().replace('{', '{{').replace('}', '}}'), DISK_TEMPLATE)
 
 
+def get_disks():
+    disks = []
+    exclude_devices = ['/dev/sr0', '/dev/sr1', '/dev/cdrom', '/dev/dvd', '/dev/fd0']
+    live_device = commands.getoutput("findmnt -n -o source /run/live/medium").split('\n')[0]
+    live_device = re.sub('[0-9]+$', '', live_device) # remove partition numbers if any
+    if live_device is not None and live_device.startswith('/dev/'):
+        exclude_devices.append(live_device)
+        print "Excluding %s (detected as the live device)" % live_device
+    lsblk = shell_exec('LC_ALL=en_US.UTF-8 lsblk -rindo TYPE,NAME,RM,SIZE,MODEL | sort -k3,2')
+    for line in lsblk.stdout:
+        try:
+            elements = line.strip().split(" ", 4)
+            if len(elements) < 4:
+                print "Can't parse blkid output: %s" % elements
+                continue
+            elif len(elements) < 5:
+                print "Can't find model in blkid output: %s" % elements
+                type, device, removable, size, model = elements[0], elements[1], elements[2], elements[3], elements[1]
+            else:
+                type, device, removable, size, model = elements
+            device = "/dev/" + device
+            if type == "disk" and device not in exclude_devices:
+                # convert size to manufacturer's size for show, e.g. in GB, not GiB!
+                unit_index = 'BKMGTPEZY'.index(size.upper()[-1])
+                l10n_unit = [_('B'), _('kB'), _('MB'), _('GB'), _('TB'), 'PB', 'EB', 'ZB', 'YB'][unit_index]
+                size = "%s %s" % (str(int(float(size[:-1]) * (1024/1000)**unit_index)), l10n_unit)
+                model = model.replace("\\x20", " ")
+                description = '{} ({})'.format(model.strip(), size)
+                if int(removable):
+                    description = _('Removable:') + ' ' + description
+                disks.append((device, description))
+        except Exception, detail:
+            print "Could not parse blkid output: %s (%s)" % (line, detail)
+    return disks
+
 def build_partitions(_installer):
     global installer
     installer = _installer
@@ -68,7 +111,7 @@ def build_partitions(_installer):
     if partition_setup.disks:
         installer._selected_disk = partition_setup.disks[0][0]
         print "Loading HTML string"
-        installer.partitions_browser.load_string(partition_setup.get_html(installer._selected_disk), 'text/html', 'UTF-8', 'file:///')
+        installer.partitions_browser.load_html(partition_setup.get_html(installer._selected_disk), 'file:///')
     print "Showing the partition screen"
     installer.builder.get_object("scrolled_partitions").show_all()
     installer.builder.get_object("treeview_disks").set_model(partition_setup)
@@ -82,7 +125,7 @@ def update_html_preview(selection):
     except TypeError, IndexError: return  # no disk is selected or no disk available
     if disk != installer._selected_disk:
         installer._selected_disk = disk
-        installer.partitions_browser.load_string(model.get_html(disk), 'text/html', 'UTF-8', 'file:///')
+        installer.partitions_browser.load_html(model.get_html(disk), 'file:///')
 
 def edit_partition_dialog(widget, path, viewcol):
     ''' assign the partition ... '''
@@ -185,44 +228,9 @@ class PartitionSetup(Gtk.TreeStore):
         installer.setup.partition_setup = self
         self.html_disks, self.html_chunks = {}, defaultdict(list)
 
-        def _get_attached_disks():
-            disks = []
-            exclude_devices = ['/dev/sr0', '/dev/sr1', '/dev/cdrom', '/dev/dvd', '/dev/fd0']
-            live_device = commands.getoutput("findmnt -n -o source /lib/live/mount/medium").split('\n')[0]
-            live_device = re.sub('[0-9]+$', '', live_device) # remove partition numbers if any
-            if live_device is not None and live_device.startswith('/dev/'):
-                exclude_devices.append(live_device)
-                print "Excluding %s (detected as the live device)" % live_device
-            lsblk = shell_exec('LC_ALL=en_US.UTF-8 lsblk -rindo TYPE,NAME,RM,SIZE,MODEL | sort -k3,2')
-            for line in lsblk.stdout:
-                try:
-                    elements = line.strip().split(" ", 4)
-                    if len(elements) < 4:
-                        print "Can't parse blkid output: %s" % elements
-                        continue
-                    elif len(elements) < 5:
-                        print "Can't find model in blkid output: %s" % elements
-                        type, device, removable, size, model = elements[0], elements[1], elements[2], elements[3], elements[1]
-                    else:
-                        type, device, removable, size, model = elements
-                    device = "/dev/" + device
-                    if type == "disk" and device not in exclude_devices:
-                        # convert size to manufacturer's size for show, e.g. in GB, not GiB!
-                        unit_index = 'BKMGTPEZY'.index(size.upper()[-1])
-                        l10n_unit = [_('B'), _('kB'), _('MB'), _('GB'), _('TB'), 'PB', 'EB', 'ZB', 'YB'][unit_index]
-                        size = "%s %s" % (str(int(float(size[:-1]) * (1024/1000)**unit_index)), l10n_unit)
-                        model = model.replace("\\x20", " ")
-                        description = '{} ({})'.format(model.strip(), size)
-                        if int(removable):
-                            description = _('Removable:') + ' ' + description
-                        disks.append((device, description))
-                except Exception, detail:
-                    print "Could not parse blkid output: %s (%s)" % (line, detail)
-            return disks
-
         os.popen('mkdir -p ' + TMP_MOUNTPOINT)
         installer.setup.gptonefi = is_efi_supported()
-        self.disks = _get_attached_disks()
+        self.disks = get_disks()
         print 'Disks: ', self.disks
         already_done_full_disk_format = False
         for disk_path, disk_description in self.disks:
@@ -243,10 +251,10 @@ class PartitionSetup(Gtk.TreeStore):
                     installer.window.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
                     print "Performing a full disk format"
                     if not already_done_full_disk_format:
-                        assign_mount_format = self.full_disk_format(disk_device)
+                        assign_mount_format = full_disk_format(disk_device)
                         already_done_full_disk_format = True
                     else:
-                        self.full_disk_format(disk_device) # Format but don't assign mount points
+                        full_disk_format(disk_device) # Format but don't assign mount points
                     installer.window.get_window().set_cursor(None)
                     print "Done full disk format"
                     disk = parted.Disk(disk_device)
@@ -316,43 +324,69 @@ class PartitionSetup(Gtk.TreeStore):
         else:
             return ""
 
-    def full_disk_format(self, device):
-        # Create a default partition set up
-        disk_label = ('gpt' if device.getLength('B') > 2**32*.9 * device.sectorSize  # size of disk > ~2TB
-                               or installer.setup.gptonefi
-                            else 'msdos')
-        separate_home_partition = device.getLength('GB') > 61
-        mkpart = (
-            # (condition, mount_as, format_as, mkfs command, size_mb)
-            # EFI
-            (installer.setup.gptonefi, EFI_MOUNT_POINT, 'vfat', 'mkfs.vfat {} -F 32 ', 300),
-            # swap - equal to RAM for hibernate to work well (but capped at ~8GB)
-            (True, SWAP_MOUNT_POINT, 'swap', 'mkswap {}', min(8800, int(round(1.1/1024 * int(getoutput("awk '/^MemTotal/{ print $2 }' /proc/meminfo")), -2)))),
-            # root
-            (True, '/', 'ext4', 'mkfs.ext4 -F {}', 30000 if separate_home_partition else 0),
-            # home
-            (separate_home_partition, '/home', 'ext4', 'mkfs.ext4 -F {}', 0),
-        )
-        run_parted = lambda cmd: os.system('parted --script --align optimal {} {} ; sync'.format(device.path, cmd))
-        run_parted('mklabel ' + disk_label)
-        start_mb = 2
-        partition_number = 0
-        for partition in mkpart:
-            if partition[0]:
-                partition_number = partition_number + 1
-                mkfs = partition[3]
-                size_mb = partition[4]
-                end = '{}MB'.format(start_mb + size_mb) if size_mb else '100%'
-                mkpart_cmd = 'mkpart primary {}MB {}'.format(start_mb, end)
-                print mkpart_cmd
-                run_parted(mkpart_cmd)
-                mkfs = mkfs.format("%s%d" % (device.path, partition_number))
-                print mkfs
-                os.system(mkfs)
-                start_mb += size_mb + 1
-        if installer.setup.gptonefi:
-            run_parted('set 1 boot on')
-        return ((i[1], i[2]) for i in mkpart if i[0])
+@idle
+def show_error(message):
+    from frontend.gtk_interface import ErrorDialog
+    ErrorDialog(_("Installer"), message)
+
+def full_disk_format(device, create_boot=False, create_swap=True):
+    # Create a default partition set up
+    disk_label = ('gpt' if device.getLength('B') > 2**32*.9 * device.sectorSize  # size of disk > ~2TB
+                           or installer.setup.gptonefi
+                        else 'msdos')
+    return_code = os.system("parted -s %s mklabel %s" % (device.path, disk_label))
+    if return_code != 0:
+        show_error(_("The partition table couldn't be written for %s. Restart the computer and try again.") % device.path)
+        Gtk.main_quit()
+        sys.exit(1)
+
+    mkpart = (
+        # (condition, mount_as, format_as, mkfs command, size_mb)
+        # EFI
+        (installer.setup.gptonefi, EFI_MOUNT_POINT, 'vfat', 'mkfs.vfat {} -F 32 ', 300),
+        # boot
+        (create_boot, '/boot', 'ext4', 'mkfs.ext4 -F {}', 1024),
+        # swap - equal to RAM for hibernate to work well (but capped at ~8GB)
+        (create_swap, SWAP_MOUNT_POINT, 'swap', 'mkswap {}', min(8800, int(round(1.1/1024 * int(getoutput("awk '/^MemTotal/{ print $2 }' /proc/meminfo")), -2)))),
+        # root
+        (True, '/', 'ext4', 'mkfs.ext4 -F {}', 0),
+    )
+    run_parted = lambda cmd: os.system('parted --script --align optimal {} {} ; sync'.format(device.path, cmd))
+    start_mb = 2
+    partition_number = 0
+    partition_prefix = ""
+    if device.path.startswith("/dev/nvme"):
+        partition_prefix = "p"
+    for partition in mkpart:
+        if partition[0]:
+            partition_number = partition_number + 1
+            mkfs = partition[3]
+            size_mb = partition[4]
+            end = '{}MB'.format(start_mb + size_mb) if size_mb else '100%'
+            mkpart_cmd = 'mkpart primary {}MB {}'.format(start_mb, end)
+            print mkpart_cmd
+            run_parted(mkpart_cmd)
+            partition_path = "%s%s%d" % (device.path, partition_prefix, partition_number)
+            num_tries = 0
+            while True:
+                if os.path.exists(partition_path):
+                    break
+                if num_tries < 5:
+                    num_tries += 1
+                    print ("Could not find %s, waiting 1s..." % partition_path)
+                    os.system("sync")
+                    time.sleep(1)
+                else:
+                    show_error(_("The partition %s could not be created. The installation will stop. Restart the computer and try again.") % partition_path)
+                    Gtk.main_quit()
+                    sys.exit(1)
+            mkfs = mkfs.format(partition_path)
+            print mkfs
+            os.system(mkfs)
+            start_mb += size_mb + 1
+    if installer.setup.gptonefi:
+        run_parted('set 1 boot on')
+    return ((i[1], i[2]) for i in mkpart if i[0])
 
 
 def to_human_readable(size):
@@ -524,6 +558,8 @@ class PartitionDialog(object):
         self.builder.get_object("label_partition_value").set_label(path)
         self.builder.get_object("label_use_as").set_markup(_("Format as:"))
         self.builder.get_object("label_mount_point").set_markup(_("Mount point:"))
+        self.builder.get_object("button_cancel").set_label(_("Cancel"))
+        self.builder.get_object("button_ok").set_label(_("OK"))
         # Build supported filesystems list
         filesystems = ['', 'swap']
         for path in ["/bin", "/sbin"]:
@@ -539,7 +575,7 @@ class PartitionDialog(object):
         # Build list of pre-provided mountpoints
         combobox = self.builder.get_object("comboboxentry_mount_point")
         model = Gtk.ListStore(str, str)
-        for i in ["/", "/home", "/boot", "/boot/efi", "/srv", "/tmp", "swap"]:
+        for i in ["/", "/@", "/home", "/@home", "/boot", "/boot/efi", "/srv", "/tmp", "swap"]:
             model.append(["", i])
         combobox.set_model(model)
         combobox.set_entry_text_column(1)
@@ -557,4 +593,4 @@ class PartitionDialog(object):
             response_is_ok = True
         else:
             response_is_ok = False
-        return response_is_ok, mount_as, format_as
\ No newline at end of file
+        return response_is_ok, mount_as, format_as
